package com.app;

import org.apache.solr.client.solrj.SolrQuery;
import org.apache.solr.client.solrj.impl.HttpSolrClient;
import org.apache.solr.client.solrj.impl.XMLResponseParser;
import org.apache.solr.client.solrj.response.QueryResponse;
import org.apache.solr.common.SolrDocument;
import org.apache.solr.common.SolrDocumentList;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import java.sql.*;

/**
 * @author Sarvesh Kumar
 * Provide solr search services.
 */

@RestController
@SpringBootApplication
public class SolrController {

    static String host;
    static String port;

    public static void main(String... args) {
        host = args != null && args.length>=1 ? args[0] : "localhost";
        port = args != null && args.length>=2 ? args[1] : "8983";
        SpringApplication.run(SolrController.class, args);
    }

    @PostMapping("/search")
    public String executeQuery(@RequestParam String column, @RequestParam String condition, @RequestParam String data) {
        System.out.println("# Query: "+column+", "+condition+", "+data);
        try {

            String urlString = "http://"+host+":"+port+"/solr/books";
            HttpSolrClient solr = new HttpSolrClient.Builder(urlString).build();
            solr.setParser(new XMLResponseParser());

            SolrQuery query = new SolrQuery();

            switch (condition) {
                case "equal" :
                    query.set("q", column+":"+data);
                    break;
                case "not-equal":
                    query.set("q", "-"+column+":"+data);
                    break;
            }

            QueryResponse response = solr.query(query);

            SolrDocumentList docList = response.getResults();

            StringBuffer sb = new StringBuffer("");
            sb.append("<table class='table table-bordered table-hover table-responsive'><thead><tr class='table-info'>");
            sb.append("<th>BookID</th><th>Title</th><th>Authors</th><th>Average Rating</th><th>Language Code</th><th>Publication Date</th><th>Publisher</th>");
            sb.append("</tr></thead><tbody>");

            for (SolrDocument doc : docList) {
                sb.append("<tr>");
                sb.append("<td>" + doc.getFieldValue("bookID") + "</td>");
                sb.append("<td>" + doc.getFieldValue("title") + "</td>");
                sb.append("<td>" + doc.getFieldValue("authors") + "</td>");
                sb.append("<td>" + doc.getFieldValue("average_rating") + "</td>");
                sb.append("<td>" + doc.getFieldValue("language_code") + "</td>");
                sb.append("<td>" + doc.getFieldValue("publication_date") + "</td>");
                sb.append("<td>" + doc.getFieldValue("publisher") + "</td>");
                sb.append("</tr>");
            }

            sb.append("</tbody></table>");

            return sb.toString();
        } catch (Exception ex) {
            return "Exception: " + ex.getMessage();
        }
    }


}
